<?php
/**
 * Pocket Project
 * 
 * @copyright 2006 Power-Station co., Ltd, ALL Rights Reserved,
 */

/**
 *  記事情報の取得
 *
 *  @access public
 *  @param  integer $post_id            記事ID
 */
function db_get_post_info($post_id)
{
    //////////////////////////////////////////////
    // MySQL4.0.x 対応のためサブクエリは使用しない
    //////////////////////////////////////////////
    $sql = 'SELECT p.post_id, p.author_name, p.subject, p.category_id, c.category_name, p.is_open, p.allow_comment,' .
            ' p.trackback_url, p.body, p.r_datetime, p.m_datetime' .
            ' FROM pb_posts AS p' .
            ' LEFT JOIN pb_categories AS c' .
                ' ON p.category_id = c.category_id' .
            ' WHERE p.post_id = ?';
    $params = array(
        intval($post_id)
    );
    $row = pocket_db_getRow($sql, $params);
    if ($row)
    {
        $row['comment_count'] = db_get_post_comments_count($row['post_id']);
        $row['trackback_count'] = db_get_post_trackbacks_count($row['post_id']);
    }
    return $row;
}

/**
 *  記事のコメント件数の取得
 *
 *  @access public
 *  @param  integer $post_id            記事ID
 */
function db_get_post_comments_count($post_id)
{
    $sql = 'SELECT count(post_comment_id) AS comment_count' .
            ' FROM pb_post_comments' .
            ' WHERE is_approved = ?' .
            ' AND post_id = ?';
    $params = array(
        intval(true),
        intval($post_id)
    );
    return pocket_db_getOne($sql, $params);
}

/**
 *  記事のトラックバック件数の取得
 *
 *  @access public
 *  @param  integer $post_id            記事ID
 */
function db_get_post_trackbacks_count($post_id)
{
    $sql = 'SELECT count(trackback_id) AS trackback_count' .
            ' FROM pb_trackbacks' .
            ' WHERE is_approved = ?' .
            ' AND post_id = ?';
    $params = array(
        intval(true),
        intval($post_id)
    );
    return pocket_db_getOne($sql, $params);
}

/**
 *  最新の記事一覧情報の取得
 *
 *  @access public
 *  @param  integer $page               ページ番号
 *  @param  integer $page_size          ページサイズ
 */
function db_get_posts4latest_post_list($page = 1, $page_size = 5)
{
    //////////////////////////////////////////////
    // MySQL4.0.x 対応のためサブクエリは使用しない
    //////////////////////////////////////////////
    $sql = 'SELECT post_id, author_name, subject, r_datetime, m_datetime' .
            ' FROM pb_posts' .
            ' WHERE is_open = ?' .
            ' ORDER BY r_datetime DESC';
    $params = array(
        intval(true)
    );
    $list = pocket_db_getAllPage($sql, $page, $page_size, $params);
    foreach ($list as $key => $item)
    {
        $list[$key]['comment_count'] = db_get_post_comments_count($item['post_id']);
        $list[$key]['trackback_count'] = db_get_post_trackbacks_count($item['post_id']);
    }
    return $list;
}

/**
 *  最新の記事一覧情報（RSS用）の取得
 *
 *  @access public
 *  @param  integer $page_size          ページサイズ
 */
function db_get_posts4latest_post_rss($page_size = 5)
{
    $sql = 'SELECT post_id, subject AS title, body AS description, r_datetime' .
            ' FROM pb_posts' .
            ' WHERE is_open = ?' .
            ' ORDER BY r_datetime DESC';
    $params = array(
        intval(true)
    );
    return pocket_db_getAllPage($sql, 1, $page_size, $params);
}

/**
 *  アーカイブ年月一覧情報の取得
 *
 *  @access public
 */
function db_get_posts4archive_list()
{
    // DB種別の判定
    if (pocket_db_isMySQL())
    {
        $sql = 'SELECT YEAR(r_datetime) as r_year, MONTH(r_datetime) as r_month, count(post_id) as post_count';
    }
    else {
        $sql = 'SELECT EXTRACT(YEAR FROM r_datetime) as r_year, EXTRACT(MONTH FROM r_datetime) as r_month, count(post_id) as post_count';
    }

    $sql .= ' FROM pb_posts' .
            ' WHERE is_open = ?' .
            ' GROUP BY r_year, r_month' .
            ' ORDER BY r_year DESC, r_month DESC';
    $params = array(
        intval(true)
    );
    return pocket_db_getAll($sql, $params);
}

/**
 *  最新の記事情報の取得
 *
 *  @access public
 *  @param  integer $page               ページ番号
 *  @param  integer $page_size          ページサイズ
 */
function db_get_posts4latest($page = 1, $page_size = 5)
{
    //////////////////////////////////////////////
    // MySQL4.0.x 対応のためサブクエリは使用しない
    //////////////////////////////////////////////
    $sql = 'SELECT p.post_id, p.author_name, p.category_id, ca.category_name, p.subject, p.body, p.r_datetime, p.m_datetime' .
            ' FROM pb_posts AS p' .
            ' LEFT JOIN pb_categories AS ca' .
                ' ON p.category_id = ca.category_id' .
            ' WHERE p.is_open = ?' .
            ' ORDER BY p.r_datetime DESC';
    $params = array(
        intval(true)
    );
    $list = pocket_db_getAllPage($sql, $page, $page_size, $params);
    foreach ($list as $key => $item)
    {
        $list[$key]['comment_count'] = db_get_post_comments_count($item['post_id']);
        $list[$key]['trackback_count'] = db_get_post_trackbacks_count($item['post_id']);
    }
    return $list;
}

/**
 *  カテゴリの記事情報の取得
 *
 *  @access public
 *  @param  integer $category_id        カテゴリID
 *  @param  integer $page               ページ番号
 *  @param  integer $page_size          ページサイズ
 */
function db_get_posts4category($category_id, $page = 1, $page_size = 5)
{
    //////////////////////////////////////////////
    // MySQL4.0.x 対応のためサブクエリは使用しない
    //////////////////////////////////////////////
    $sql = 'SELECT post_id, author_name, subject, body, r_datetime, m_datetime' .
            ' FROM pb_posts' .
            ' WHERE is_open = ?' .
            ' AND category_id = ?' .
            ' ORDER BY r_datetime DESC';
    $params = array(
        intval(true),
        intval($category_id)
    );
    $list = pocket_db_getAllPage($sql, $page, $page_size, $params);
    foreach ($list as $key => $item)
    {
        $list[$key]['comment_count'] = db_get_post_comments_count($item['post_id']);
        $list[$key]['trackback_count'] = db_get_post_trackbacks_count($item['post_id']);
    }
    return $list;
}

/**
 *  カテゴリの記事件数の取得
 *
 *  @access public
 *  @param  integer $category_id        カテゴリID
 */
function db_get_posts_count4category($category_id)
{
    $sql = 'SELECT count(post_id)' .
            ' FROM pb_posts' .
            ' WHERE is_open = ?' .
            ' AND category_id = ?';
    $params = array(
        intval(true),
        intval($category_id)
    );
    return pocket_db_getOne($sql, $params);
}

/**
 *  キーワード検索の記事情報の取得
 *
 *  @access public
 *  @param  string  $keyword            キーワード
 *  @param  integer $page               ページ番号
 *  @param  integer $page_size          ページサイズ
 */
function db_get_posts4keyword($keyword, $page = 1, $page_size = 5)
{
    //////////////////////////////////////////////
    // MySQL4.0.x 対応のためサブクエリは使用しない
    //////////////////////////////////////////////
    $sql = 'SELECT p.post_id, p.author_name, p.category_id, ca.category_name, p.subject, p.body, p.r_datetime, p.m_datetime' .
            ' FROM pb_posts AS p' .
            ' LEFT JOIN pb_categories AS ca' .
                ' ON p.category_id = ca.category_id' .
            ' WHERE p.is_open = ?';
    $params = array(
        intval(true)
    );

    if ($keyword)
    {
        $keyword = str_replace('　', ' ', $keyword);
        $word_list = explode(' ', $keyword);
        foreach ($word_list as $word)
        {
            $sql .= ' AND (p.subject LIKE ? OR p.body LIKE ?)';
            $word = poket_db_replaceLikeWord($word);
            $params[] = '%'.$word.'%';
            $params[] = '%'.$word.'%';
        }
    }

    $sql .= ' ORDER BY p.r_datetime DESC';

    $list = pocket_db_getAllPage($sql, $page, $page_size, $params);
    foreach ($list as $key => $item)
    {
        $list[$key]['comment_count'] = db_get_post_comments_count($item['post_id']);
        $list[$key]['trackback_count'] = db_get_post_trackbacks_count($item['post_id']);
    }
    return $list;
}

/**
 *  キーワード検索の記事件数の取得
 *
 *  @access public
 *  @param  string  $keyword            キーワード
 */
function db_get_posts_count4keyword($keyword)
{
    $sql = 'SELECT count(post_id)' .
            ' FROM pb_posts' .
            ' WHERE is_open = ?';
    $params = array(
        intval(true)
    );

    if ($keyword)
    {
        $keyword = str_replace('　', ' ', $keyword);
        $word_list = explode(' ', $keyword);
        foreach ($word_list as $word)
        {
            $sql .= ' AND (subject LIKE ? OR body LIKE ?)';
            $word = poket_db_replaceLikeWord($word);
            $params[] = '%'.$word.'%';
            $params[] = '%'.$word.'%';
        }
    }

    return pocket_db_getOne($sql, $params);
}

/**
 *  指定年月の記事情報の取得
 *
 *  @access public
 *  @param  integer $year               年
 *  @param  integer $month              月
 *  @param  integer $page               ページ番号
 *  @param  integer $page_size          ページサイズ
 */
function db_get_posts4year_month($year, $month, $page = 1, $page_size = 5)
{
    //////////////////////////////////////////////
    // MySQL4.0.x 対応のためサブクエリは使用しない
    //////////////////////////////////////////////
    $sql = 'SELECT p.post_id, p.author_name, p.category_id, ca.category_name, p.subject, p.body, p.r_datetime, p.m_datetime' .
            ' FROM pb_posts AS p' .
            ' LEFT JOIN pb_categories AS ca' .
                ' ON p.category_id = ca.category_id' .
            ' WHERE p.is_open = ?';

    // DB種別の判定
    if (pocket_db_isMySQL())
    {
        $sql .= ' AND YEAR(p.r_datetime) = ?' .
                ' AND MONTH(p.r_datetime) = ?';
    }
    else {
        $sql .= ' AND EXTRACT(YEAR FROM p.r_datetime) = ?' .
                ' AND EXTRACT(MONTH FROM p.r_datetime) = ?';
    }

    $sql .= ' ORDER BY p.r_datetime DESC';
    $params = array(
        intval(true),
        intval($year),
        intval($month)
    );
    $list = pocket_db_getAllPage($sql, $page, $page_size, $params);
    foreach ($list as $key => $item)
    {
        $list[$key]['comment_count'] = db_get_post_comments_count($item['post_id']);
        $list[$key]['trackback_count'] = db_get_post_trackbacks_count($item['post_id']);
    }
    return $list;
}

/**
 *  指定年月の記事件数の取得
 *
 *  @access public
 *  @param  integer $year               年
 *  @param  integer $month              月
 */
function db_get_posts_count4year_month($year, $month)
{
    $sql = 'SELECT count(post_id)' .
            ' FROM pb_posts' .
            ' WHERE is_open = ?';

    // DB種別の判定
    if (pocket_db_isMySQL())
    {
        $sql .= ' AND YEAR(r_datetime) = ?' .
                ' AND MONTH(r_datetime) = ?';
    }
    else {
        $sql .= ' AND EXTRACT(YEAR FROM r_datetime) = ?' .
                ' AND EXTRACT(MONTH FROM r_datetime) = ?';
    }

    $params = array(
        intval(true),
        intval($year),
        intval($month)
    );
    return pocket_db_getOne($sql, $params);
}

/**
 *  指定日の記事情報の取得
 *
 *  @access public
 *  @param  integer $year               年
 *  @param  integer $month              月
 *  @param  integer $day                日
 */
function db_get_posts4day($year, $month, $day)
{
    //////////////////////////////////////////////
    // MySQL4.0.x 対応のためサブクエリは使用しない
    //////////////////////////////////////////////
    $sql = 'SELECT p.post_id, p.author_name, p.category_id, ca.category_name, p.subject, p.body, p.r_datetime, p.m_datetime' .
            ' FROM pb_posts AS p' .
            ' LEFT JOIN pb_categories AS ca' .
                ' ON p.category_id = ca.category_id' .
            ' WHERE p.is_open = ?';

    // DB種別の判定
    if (pocket_db_isMySQL())
    {
        $sql .= ' AND YEAR(p.r_datetime) = ?' .
                ' AND MONTH(p.r_datetime) = ?' .
                ' AND DAY(p.r_datetime) = ?';
    }
    else {
        $sql .= ' AND EXTRACT(YEAR FROM p.r_datetime) = ?' .
                ' AND EXTRACT(MONTH FROM p.r_datetime) = ?' .
                ' AND EXTRACT(DAY FROM p.r_datetime) = ?';
    }

    $sql .= ' ORDER BY p.r_datetime DESC';
    $params = array(
        intval(true),
        intval($year),
        intval($month),
        intval($day)
    );
    $list = pocket_db_getAll($sql, $params);
    foreach ($list as $key => $item)
    {
        $list[$key]['comment_count'] = db_get_post_comments_count($item['post_id']);
        $list[$key]['trackback_count'] = db_get_post_trackbacks_count($item['post_id']);
    }
    return $list;
}

/**
 *  指定年月の投稿日情報の取得
 *
 *  @access public
 *  @param  integer $year               年
 *  @param  integer $month              月
 */
function db_get_post_date4year_month($year, $month)
{
    // DB種別の判定
    if (pocket_db_isMySQL())
    {
        $sql = 'SELECT DISTINCT DAY(r_datetime) AS post_day' .
                ' FROM pb_posts' .
                ' WHERE is_open = ?' .
                ' AND YEAR(r_datetime) = ?' .
                ' AND MONTH(r_datetime) = ?' .
                ' ORDER BY post_day';
    }
    else {
        $sql = 'SELECT DISTINCT EXTRACT(DAY FROM r_datetime) AS post_day' .
                ' FROM pb_posts' .
                ' WHERE is_open = ?' .
                ' AND EXTRACT(YEAR FROM r_datetime) = ?' .
                ' AND EXTRACT(MONTH FROM r_datetime) = ?' .
                ' ORDER BY post_day';
    }

    $params = array(
        intval(true),
        intval($year),
        intval($month)
    );
    return pocket_db_getCol($sql, $params);
}

/**
 *  非公開の記事一覧情報の取得
 *
 *  @access public
 */
function db_get_posts4closed()
{
    //////////////////////////////////////////////
    // MySQL4.0.x 対応のためサブクエリは使用しない
    //////////////////////////////////////////////
    $sql = 'SELECT p.post_id, p.author_name, p.category_id, ca.category_name, p.subject, p.body, p.r_datetime, p.m_datetime' .
            ' FROM pb_posts AS p' .
            ' LEFT JOIN pb_categories AS ca' .
                ' ON p.category_id = ca.category_id' .
            ' WHERE p.is_open = ?' .
            ' ORDER BY p.r_datetime DESC';
    $params = array(
        intval(false)
    );
    $list = pocket_db_getAll($sql, $params);
    foreach ($list as $key => $item)
    {
        $list[$key]['comment_count'] = db_get_post_comments_count($item['post_id']);
        $list[$key]['trackback_count'] = db_get_post_trackbacks_count($item['post_id']);
    }
    return $list;
}

/**
 *  非公開の記事件数の取得
 *
 *  @access public
 */
function db_get_posts_count4closed()
{
    $sql = 'SELECT count(post_id)' .
            ' FROM pb_posts' .
            ' WHERE is_open = ?';
    $params = array(
        intval(false)
    );
    return pocket_db_getOne($sql, $params);
}

/**
 *  記事情報の登録
 *
 *  @access public
 *  @param  string  $subject            タイトル
 *  @param  string  $author_name        投稿者
 *  @param  integer $category_id        カテゴリID
 *  @param  integer $is_open            公開フラグ
 *  @param  integer $allow_comment      コメント許可フラグ
 *  @param  string  $trackback_url      トラックバックURL
 *  @param  string  $body               本文
 *  @param  string  $r_datetime         投稿日付
 */
function db_insert_post($author_name, $subject, $category_id, $is_open, $allow_comment, $trackback_url, $body, $r_datetime)
{
    $data = array(
        'author_name' => $author_name,
        'subject' => $subject,
        'category_id' => $category_id,
        'is_open' => $is_open,
        'allow_comment' => $allow_comment,
        'trackback_url' => $trackback_url,
        'body' => $body,
        'r_datetime' => $r_datetime,
        'm_datetime' => pocket_db_getNow()
    );
    $insertId = pocket_db_insert('pb_posts', $data, 'pb_posts_post_id_seq');

    $data = array(
        'category_id' => $category_id,
        'post_id' => $insertId
    );
    pocket_db_insert('pb_category_posts', $data, 'pb_category_posts_category_post_id_seq');

    return $insertId;
}

/**
 *  記事情報の更新
 *
 *  @access public
 *  @param  integer $post_id            ID
 *  @param  string  $author_name        投稿者
 *  @param  string  $subject            タイトル
 *  @param  integer $category_id        カテゴリID
 *  @param  integer $is_open            公開フラグ
 *  @param  integer $allow_comment      コメント許可フラグ
 *  @param  string  $trackback_url      トラックバックURL
 *  @param  string  $body               本文
 *  @param  string  $r_datetime         投稿日付
  */
function db_update_post($post_id, $author_name, $subject, $category_id, $is_open, $allow_comment, $trackback_url, $body, $r_datetime)
{
    $data = array(
        'author_name' => $author_name,
        'subject' => $subject,
        'category_id' => $category_id,
        'is_open' => $is_open,
        'allow_comment' => $allow_comment,
        'trackback_url' => $trackback_url,
        'body' => $body,
        'r_datetime' => $r_datetime,
        'm_datetime' => pocket_db_getNow()
    );
    $where = array(
        'post_id' => intval($post_id)
    );
    if (!pocket_db_update('pb_posts', $data, $where)) {return false;}

    $data = array(
        'category_id' => $category_id
    );
    $where = array(
        'post_id' => intval($post_id)
    );
    return pocket_db_update('pb_category_posts', $data, $where);
}

/**
 *  記事情報のカテゴリの更新
 *
 *  @access public
 *  @param  integer $category_id        カテゴリID
 */
function db_update_posts_category4default($category_id)
{
    $data = array(
        'category_id' => 1
    );
    $where = array(
        'category_id' => intval($category_id)
    );
    if (!pocket_db_update('pb_posts', $data, $where)) {return false;}

    $data = array(
        'category_id' => 1
    );
    $where = array(
        'category_id' => intval($category_id)
    );
    return pocket_db_update('pb_category_posts', $data, $where);
}

/**
 *  記事情報の削除
 *
 *  @access public
 *  @param  integer $post_id            ID
 */
function db_delete_post($post_id)
{
    if (!pocket_db_delete('pb_posts', 'post_id', $post_id)) {return false;}
    if (!pocket_db_delete('pb_category_posts', 'post_id', $post_id)) {return false;}
    return pocket_db_delete('pb_post_comments', 'post_id', $post_id);
}

?>
